"""Manage paper labels: TOP CIT CLI REF"""

__copyright__ = "Copyright (C) 2022-present, DV Klopfenstein, PhD. All rights reserved."
__author__ = "DV Klopfenstein, PhD"


class TopCitRef:
    """Manage paper labels: TOP CIT CLI REF"""

    label_list = [
        'TOP',  # Paper of interest
        'CIT',  # A paper (not a clinical study) citing the paper of interest
        'CLI',  # A clinical study paper citing the paper of interest
        'REF',  # A paper in the reference list of the paper of interest
    ]

    label_set = set(label_list)

    choices = label_list + ['CITS', 'ALL']

    def add_arguments(self, parser):
        """Manage paper labels arguments: TOP CIT CLI REF"""
        # pylint: disable=line-too-long
        parser.add_argument(
            '-p', metavar='labels', dest='paper_labels', type=str, nargs='*',
            default=['TOP',],
            choices=self.choices,
            help=f'Paper label choices: {" ".join(self.choices)} (default: TOP)',
        )

    def adjust_args(self, args_paper_labels):
        """Given labels and aliases (CITS, ALL), return official label names"""
        if not args_paper_labels:
            return None
        ret = set()
        arg_set = set(args_paper_labels)
        if 'ALL' in arg_set:
            ret.update(self.label_list)
            return ret
        if 'CITS' in arg_set:
            ret.add('CIT')
            ret.add('CLI')
        ret.update(arg_set.intersection(self.label_list))
        return ret


# Copyright (C) 2022-present DV Klopfenstein, PhD. All rights reserved.



"""Given a PubMed ID (PMID), return a list of citing publications"""

__copyright__ = "Copyright (C) 2019-present, DV Klopfenstein, PhD. All rights reserved."
__author__ = "DV Klopfenstein, PhD"

from pmidcite.cli.icite import NIHiCiteCli  # get_argparser
from pmidcite.cfg import get_cfgparser


def main():
    """Print NIH citation data for requested papers specified with PubMedIDs (PMIDs)"""
    NIHiCiteCli(get_cfgparser(prt=None)).cli()


# Copyright (C) 2019-present, DV Klopfenstein, PhD. All rights reserved.



"""Summarize NIH citation data for requested papers from the commandline or in files"""

from collections import namedtuple
from collections import defaultdict

__copyright__ = "Copyright (C) 2022-present, DV Klopfenstein, PhD. All rights reserved."
__author__ = "DV Klopfenstein, PhD"


class SummarizePapers:
    """Summarize NIH citation data for requested papers from the commandline or in files"""

    def __init__(self, name, nih_grouper=None):
        self.name = name
        self.nts = None
        self.num_papers_all = None
        self.nihgrpr = nih_grouper

    def str_oneline(self):
        """Get str that is a one-line summary of many papers/citiations"""
        grp2nts = self._get_stats_grpr() if self.nihgrpr else self._get_stats_nogrpr()
        years = self.get_years()
        year_min = min(years)
        year_max = max(years)
        return '{NIHP} {Ys:3} years:{Y0:4}-{Y1:4} {N:5} papers {NAME}'.format(
            NIHP=self._str_group_percs(grp2nts),
            Ys=year_max-year_min,
            Y0=year_min,
            Y1=year_max,
            N=self.num_papers_all,
            NAME=self.name)

    def get_years(self):
        """Get the years of all publications"""
        return list(nt.year for nt in self.nts)

    def _str_group_percs(self, grp2nts):
        """Get precentages of papers in each group"""
        lst = []
        for grp in ['i', '4', '3', '2', '1', '0']:
            num_papers_grp = len(grp2nts[grp]) if grp2nts else 0
            abc = '{G}={P}'.format(
                G=grp,
                P='{:05.1f}%'.format(
                    num_papers_grp/self.num_papers_all*100) if num_papers_grp != 0 else "......")
            lst.append(abc)
        return ' '.join(lst)

    def _get_stats_grpr(self):
        """Get summary information for list of papers"""
        grp2nts = defaultdict(list)
        grpr = self.nihgrpr
        for ntd in self.nts:
            grp2nts[grpr.str_group(ntd.nih_perc)].append(ntd)
            ##print('DDDDDDDD', ntd)
        return grp2nts

    def _get_stats_nogrpr(self):
        """Get summary information for list of papers"""
        grp2nts = defaultdict(list)
        for ntd in self.nts:
            grp2nts[ntd.nih_group].append(ntd)
        return grp2nts

    @staticmethod
    def read_lines(filename, top_cit_ref):
        """Read paper citation lines"""
        if top_cit_ref is None:
            top_cit_ref = {'TOP',}  # TOP, CIT, CLI, REF
        nts = []
        nto = namedtuple('iciteline', (
            'line pmid aart nih_perc nih_group year num_cite_all num_cite num_clin num_refs'))
        with open(filename) as ifstrm:
            for line in ifstrm:
                if line[:3] in top_cit_ref:
                    flds = line.split(maxsplit=10)
                    if flds[1].isdigit():
                        num_cite = int(flds[7])
                        num_clin = int(flds[8])
                        nts.append(nto(
                            line=line.rstrip(),
                            pmid=int(flds[1]),
                            aart=f'{flds[2]} {flds[3]}',
                            nih_perc=int(flds[4]),
                            nih_group=flds[5],     # -i or a number
                            year=int(flds[6]),
                            num_cite_all=num_cite + num_clin,
                            num_cite=num_cite,
                            num_clin=num_clin,
                            num_refs=int(flds[9])))
        return nts

    # -- Constructors ------------------------------------------------------------
    @classmethod
    def from_file(cls, filename, nih_grouper=None, top_cit_ref=None):
        """Get SummarizePapers instance, given a file filled with icite lines w/TOP|CIT|CLI|REF"""
        obj = cls(filename, nih_grouper)
        obj.nts = obj.read_lines(filename, top_cit_ref)
        obj.num_papers_all = len(obj.nts)
        return obj


# Copyright (C) 2022-present, DV Klopfenstein, PhD. All rights reserved.



"""Summarize NIH citation data for requested papers from the commandline or in files"""

from sys import stdout
from argparse import ArgumentParser
from pmidcite.cli.utils import prt_loc_rcfile
from pmidcite.cli.utils import get_files_exists
from pmidcite.summarize_papers import SummarizePapers
from pmidcite.icite.top_cit_ref import TopCitRef

__copyright__ = "Copyright (C) 2022-present, DV Klopfenstein, PhD. All rights reserved."
__author__ = "DV Klopfenstein, PhD"


class SummarizePapersCli:
    """Summarize NIH citation data for requested papers from the commandline or in files"""

    def __init__(self, cfg):
        self.cfg = cfg
        self.topcitref = TopCitRef()

    def get_argparser(self):
        """Argument parser for summarizing the citations on set(s) of papers"""
        parser = ArgumentParser(
            description="Summarize NIH's citation data on a set(s) of papers",
            add_help=False)
        ##cfg = self.cfg
        # https://docs.python.org/3/library/argparse.html
        # https://docs.python.org/3/library/argparse.html#action
        # - PMIDs ----------------------------------------------------------------------------
        parser.add_argument(
            '-h', '--help', action='store_true',
            help='print this help message and exit (also --help)')
        parser.add_argument(
            'files', metavar='FILES', type=str, nargs='*',
            help='File(s) containing NIH citation data for numerous papers with PMIDs')
        ##parser.add_argument(
        ##    '-i', '--infile', nargs='*',
        ##    help='Files containing NIH citation data for numerous papers with PMIDs')
        ##parser.add_argument(
        ##    '-o', '--outfile',
        ##    help='Write current citation report to an ASCII text file.')
        ##parser.add_argument(
        ##    '-f', '--force_write', action='store_true',
        ##    help='if an existing outfile file exists, overwrite it.')

        self.cfg.get_nihgrouper().add_arguments(parser)
        ##parser.add_argument(
        ##    '--md', action='store_true',
        ##    help='Print using markdown table format.')
        parser.add_argument(
            '--print-rcfile', action='store_true',
            help='Print the location of the pmidcite configuration file (env var: PMIDCITECONF)')
        self.topcitref.add_arguments(parser)
        return parser

    def cli(self):
        """Run citation summary on a set(s) of PMIDs"""
        argparser = self.get_argparser()
        args = argparser.parse_args()
        ##print('ARGS CITE SUMMARY ../pmidcite/src/pmidcite/cli/summarize_papers.py', args)
        if args.print_rcfile:
            prt_loc_rcfile(self.cfg, stdout)
        files = get_files_exists(args.files, stdout)
        if args.help or not files:
            argparser.print_help()
            ##print(f'\nHelp message printed because: -h or --help == {args.help} or {args.files}')
        nih_grouper = self.cfg.get_nihgrouper(args.min1, args.min2, args.min3, args.min4)
        self._summarize_papers(files, nih_grouper, self.topcitref.adjust_args(args.paper_labels))
        if args.prt_nihgrpr:
            print(nih_grouper)

    @staticmethod
    def _summarize_papers(files, nih_grouper, top_cit_refs):
        """Summarize papers"""
        for filename in files:
            sumpap = SummarizePapers.from_file(
                filename=filename,
                nih_grouper=nih_grouper,
                top_cit_ref=top_cit_refs)
            print(sumpap.str_oneline())


# Copyright (C) 2022-present, DV Klopfenstein, PhD. All rights reserved.



#!/usr/bin/env python3
"""Summarize NIH citation data for requested papers from the commandline or in files"""

__copyright__ = "Copyright (C) 2022-present, DV Klopfenstein, PhD. All rights reserved."
__author__ = "DV Klopfenstein, PhD"

from pmidcite.cli.summarize_papers import SummarizePapersCli # get_argparser
from pmidcite.cfg import get_cfgparser


def main():
    """Summarize NIH citation data for requested papers from the commandline or in files"""
    SummarizePapersCli(get_cfgparser(prt=None)).cli()

if __name__ == '__main__':
    main()

# Copyright (C) 2022-present, DV Klopfenstein, PhD. All rights reserved.
